/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.TimerTask;

import org.eclipse.core.runtime.Status;

import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationAuthority;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationTaskException;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.dav.daf.main.config.management.ConfigAreaAndVersion;
import de.bsvrz.dav.daf.main.config.management.ConfigurationControl;
import de.bsvrz.dav.daf.main.config.management.consistenycheck.ConsistencyCheckResultInterface;
import de.bsvrz.puk.config.configFile.datamodel.ConfigConfigurationArea;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Mit dieser Klasse wird die Konfiguration gestartet. Die Konfiguration kann in folgenden verschiedenen Arbeitsmodi
 * parallel gestartet werden:
 * <ul>
 * <li>Import von Versorgungsdateien</li>
 * <li>Export von Versorgungsdateien</li>
 * <li>Konsistenzprfung der Konfiguration</li>
 * <li>Aktivierung von Konfigurationsbereichen</li>
 * <li>Freigabe zur bernahme von Konfigurationsbereichen</li>
 * <li>Freigabe zur Aktivierung von Konfigurationsbereichen</li>
 * <li>Konfigurationseditor zur interaktiven Versorgung</li>
 * <li>Konfiguration als dienstleistender Hintergrundproze fr andere Applikationen des Systems</li>
 * </ul>
 * <p/>
 * Die Konfiguration kann mit folgenden spezifischen Aufrufparametern gestartet werden:
 * <ul>
 * <li>-verwaltung=datei</li>ber diesen Aufrufparameter wird die Verwaltungsdatei spezifiziert, aus der sich die
 * Konfiguration initialisieren soll. Wenn keine weiteren Aufrufparameter angegeben werden, dann wird der Arbeitsmodus
 * 'Konfiguration' gestartet.
 * <ul>
 * Soll Die Konfiguration im Arbeitsmodus 'Konfiguration' betrieben werden, so knnen/mssen folgende Parameter
 * angegeben werden:<br>
 * <li>Optionaler Parameter: -benutzerverwaltung=datei</li>ber diesen Parameter wird die Datei spezifiziert, die die
 * Benutzerverwaltungsinformationen(Benutzernamen, Passwrter, usw.) enthlt. Wird keine Datei angegeben, wird eine
 * Default-Datei benutzt. Neue Benutzer, nderungen der Passwrter, usw. werden in der angegebenen Datei durchgefhrt.<br>
 * <li>-benutzer=Benutzername</li> Enthlt den Benutzernamen, mit dem sich die Konfiguration beim Datenverteiler
 * anmeldet
 * <li>-authentifizierung=datei</li> Enthlt das Passwort, mit dem sich die Konfiguration beim Datenverteiler anmeldet
 * </ul>
 * <li>-import=pid,pid,...</li>ber diesen Aufrufparameter werden eine oder mehrere Pids der Konfigurationsbereiche
 * angegeben, die importiert werden sollen. Wenn dieser Aufrufparameter angegeben wurde, dann wird der Arbeitsmodus
 * 'Import' gestartet. In diesem Arbeitsmodus mssen auerdem die Aufrufparameter '-verzeichnis' und '-verwaltung'
 * angegeben werden.
 * <li>-export=pid,pid,...</li>ber diesen Aufrufparameter werden eine oder mehrere Pids der Konfigurationsbereiche
 * angegeben, die exportiert werden sollen. Wenn dieser Aufrufparameter angegeben wurde, dann wird der Arbeitsmodus
 * 'Export' gestartet. In diesem Arbeitsmodus mssen auerdem die Aufrufparameter '-verzeichnis' und '-verwaltung'
 * angegeben werden.
 * <li>-verzeichnis=verzeichnis</li>Dieser Aufrufparameter gilt im Zusammenhang mit dem Arbeitsmodus 'Import' oder
 * 'Export'. Er gibt an, wo (beim Import) sich die Dateien befinden bzw. wohin (beim Export) die Dateien geschrieben
 * werden sollen.
 * <li>-editor</li>ber diesen Aufrufparameter wird spezifiziert, dass der Arbeitsmodus 'Konfigurationseditor' gestartet
 * werden soll. Wenn zustzlich der Aufrufparameter '-verwaltung' angegeben wird, dann arbeitet der Editor im
 * Offline-Modus direkt auf den Konfigurationsdateien. Ansonsten arbeitet der Editor im Online-Modus und versucht ber
 * die Datenverteilerschnittstelle eine Verbindung zu einer Instanz im Arbeitsmodus 'Konfiguration' aufzubauen.
 * <li>-konsistenzprfung</li>Beauftragt die Konfiguration alle Konfigurationsbereiche in der aktuellsten Version auf
 * Konsistenz zu prfen. In diesem Arbeitsmodus muss auerdem der Aufrufparameter '-verwaltung' angegeben werden.
 * <li>-aktivierung</li>Beauftragt die Konfiguration alle Konfigurationsbereiche in der aktuellsten Version zu
 * aktivieren. In diesem Arbeitsmodus muss auerdem der Aufrufparameter '-verwaltung' angegeben werden.
 * <li>-freigabeaktivierung</li>Beauftragt die Konfiguration alle Konfigurationsbereiche in der aktuellsten Version fr
 * andere Konfigurationen "zur Aktivierung freizugeben". In diesem Arbeitsmodus muss auerdem der Aufrufparameter
 * '-verwaltung' angegeben werden.
 * <li>-freigabebernahme</li>Beauftragt die Konfiguration alle Konfigurationsbereiche in der aktuellsten Version zur
 * bernahme freizugeben. In diesem Arbeitsmodus muss auerdem der Aufrufparameter '-verwaltung' angegeben werden.
 * <li>-freigabeZurAktivierungOhneLokaleAktivierungDurchDenKV=pid,pid,...</li>ber diesen Aufrufparameter werden eine
 * oder mehrere Pids von Konfigurationsbereichen angegeben. Diese Bereiche knnen durch eine andere Konfiguration
 * aktiviert werden, ohne dass der Konfigurationsverantwortliche der Bereiche die angegebene Bereiche aktiviert hat. In
 * diesem Arbeitsmodus muss auerdem der Aufrufparameter '-verwaltung' angegeben werden.
 * <li>keine Aufrufparameter</li>Ohne Aufrufparameter wird der Arbeitsmodus 'Konfigurationseditor' im Online-Modus
 * gestartet. </ul>
 * 
 * @author Kappich Systemberatung
 * @version $Revision: 1.19 $
 */

public class KonfigApplikation
{

  /** DebugLogger fr Debug-Ausgaben */
  private static Debug _debug;

  // /** Thread, der beim beenden oder nach einer festen Zeitspanne, die Konfiguration auffordert zu speichern. */
  // private AutoSicherung _autoSaver = null;

  /** Thread, der beim Herunterfahren des Systems die Daten sichert. */
  private AutomatischesFreigeben _autoCloser = null;

  /**
   * Zeitspanne, die gewartet wird bis die Konfiguration die gepufferten Daten persistent speichert.
   */
  public static final long _bufferTime = 10 * 60 * 1000;

  private DataModel _dataModel;

  // /**
  // * Main-Methode, welche die Aufrufparameter entgegennimmt und den Konstruktor aufruft.
  // *
  // * @param args die Aufrufparameter dieser Klasse
  // */
  // public static void main(String[] args) {
  // new ConfigurationApp(args);
  // }

  /**
   * Der Konstruktor wertet die Aufrufargumente aus.
   * 
   * @param args
   *          die Aufrufargumente
   */
  public KonfigApplikation(DataModel dataModel)
  {

    if (dataModel != null)
    {
      _dataModel = dataModel;
    }
    else
    {

      String meldung = "Datemodell ist null";

      IllegalArgumentException e = new IllegalArgumentException();
      Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, Status.ERROR, meldung, e));
    }
  }

  // /**
  // * Diese Methode setzt alle dynamischen Objekte, die als "ungltigBeimNeustart" markiert sind, auf ungltig.
  // *
  // * @param dataModel Datenmodell
  // */
  // private void dealWithInvalidOnRestartObjects(DataModel dataModel) throws ConfigurationChangeException {
  //
  // // Alle Objekte/Elemente des Typs typ.dynamischerTyp. Die entspricht allen Typen, die dynamisch sind.
  // final List<SystemObject> allDynamicTypes = dataModel.getType("typ.dynamischerTyp").getElements();
  // // Auf die richtige Klasse casten
  // final List<SystemObjectType> systemObjectTypes = new ArrayList<SystemObjectType>(allDynamicTypes.size());
  // for(SystemObject systemObject : allDynamicTypes) {
  // systemObjectTypes.add((DynamicObjectType)systemObject);
  // }
  //
  // // Alle Bereiche, die der Konfigurationsverantwortliche der Konfiguration ndern darf
  // final Collection<ConfigurationArea> examineAreas = getAuthorityConfigAreas(dataModel.getConfigurationAuthority(),
  // dataModel);
  //
  // // Alle aktuellen dynamischen Objekte, fr die der Konfigurationsverantwortliche der Konfiguration
  // // verantwortlich ist. Achtung, diese Methode benutzt die Vererbung, eine Einschrnkung der Tyen macht also
  // // keinen Sinn !!
  // final Collection<SystemObject> allDynamicObjects = dataModel.getObjects(examineAreas, systemObjectTypes,
  // ObjectTimeSpecification.valid());
  //
  // // Alle Objekte auf ungltig setzen, die transient sind.
  // for(SystemObject systemObject : allDynamicObjects) {
  //
  // // Objekttyp des Objekts anfordern, dies kann nur ein dynamischer Typ sein
  // final DynamicObjectType dynamicObjectType = (DynamicObjectType)systemObject.getType();
  //
  // if(dynamicObjectType.getPersistenceMode() == DynamicObjectType.PersistenceMode.PERSISTENT_AND_INVALID_ON_RESTART) {
  // systemObject.invalidate();
  // _debug.finest("InvalidOnRestart Objekt: " + systemObject.getPidOrNameOrId());
  // }
  // }
  // // _debug.info("Anzahl gefundener transienter dynamischer Typen: " + transientObjectTypes.size() +
  // " Anzahl dynamischer Objekte, die transient waren und auf ungltig gesetzt wurden: " + allDynamicObjects.size());
  // }

  /**
   * Diese Methode sucht zu einem Konfigurationsverantwortlichen alle aktiven Konfigurationsbereiche heraus, fr die er
   * verantwortlich ist.
   * 
   * @param authority
   *          Verantwortlicher, zu dem die Bereiche gesucht werden sollen
   * @param dataModel
   *          Datenmodell, in dem die Bereiche vorhanden sind
   */
  private Collection<ConfigurationArea> getKVKonfigBereiche(ConfigurationAuthority authority)
  {
    final Collection<ConfigurationArea> resultAreas = new ArrayList<ConfigurationArea>();
    // alle aktiven Konfigurationsbereiche ermitteln
    final SystemObjectType objectType = _dataModel.getType("typ.konfigurationsBereich");
    final List<SystemObject> objects = objectType.getElements();
    for (SystemObject systemObject : objects)
    {
      final ConfigurationArea configurationArea = (ConfigurationArea)systemObject;
      if (authority.equals(configurationArea.getConfigurationAuthority()))
      {
        resultAreas.add(configurationArea);
      }
    }
    return resultAreas;
  }

  /**
   * Die eingelesenen Strings werden von Leerzeichen befreit, die vor oder nach den Pids stehen.
   * 
   * @param pids
   *          die zu berprfenden Strings
   * 
   * @return die Strings enthalten jetzt vor oder nach der Pid keine Leerzeichen mehr
   */
  private List<String> trimPids(String[] pids)
  {
    final List<String> pidList = new ArrayList<String>();
    for (String pid : pids)
    {
      pidList.add(pid.trim());
    }
    return pidList;
  }

  // private void startConfigurationEditorOnline() {
  // _debug.info("Konfigurationseditor im Online-Modus wird gestartet");
  // }
  //
  // private void startConfigurationEditorOffline(File managementFile) {
  // _debug.info("Konfigurationseditor im Offline-Modus wird gestartet: " + managementFile.getName());
  // }

  // /** Startet die Konfiguration */
  // private DataModel startConfiguration(File managementFile, File userManagementFile, ClientDavParameters
  // dafParameters)
  // throws CommunicationError, ConnectionException, ParserConfigurationException, MissingParameterException,
  // InterruptedException, InconsistentLoginException {
  // _debug.info("Konfiguration wird gestartet: " + managementFile.getName());
  // // Datenmodell starten
  // final DataModel dataModel = new ConfigDataModel(managementFile);
  //
  // // Startet den Mechanismus, der Konfigurationsanfragen entgegennimmt und verarbeitet
  // final ConfigurationCommunicator configurationCommunicator;
  // try {
  // configurationCommunicator = new ConfigurationCommunicator(dataModel, userManagementFile, dafParameters);
  // }
  // catch(RuntimeException e) {
  // // Im Fehlerfall dafr sorgen, dass die bereits angelegten Lock-Dateien gelscht werden
  // ((ConfigDataModel)dataModel).close();
  // throw e;
  // }
  //
  // // Automatisches Speichern aktivieren
  // startAutoSaver(dataModel);
  // // Beim herunterfahren des Systems speichern
  // startAutoCloser(dataModel, configurationCommunicator);
  //
  // _debug.info("Konfiguration", dataModel);
  // return dataModel;
  // }

  /**
   * @param dataModel
   * @param importPath
   * @param pids
   * @throws ConfigurationChangeException
   */
  public void startImport(File importPath, List<String> pids) throws ConfigurationChangeException
  {
    // Alle Dateien sind fr andere gesperrt worden (lock-Dateien). Diese mssen wieder freigegeben werden, sobald
    // die Aktion beendet wurde.

    // starteAutomatischesBeenden();

    // Import starten
    // try
    {
      ((ConfigurationControl)_dataModel).importConfigurationAreas(importPath, pids);
    }
    // catch(Exception ex) {
    // String meldung = "Der Import konnte nicht ordnungsgem durchgefhrt werden";
    // RuntimeException e = new RuntimeException(meldung, ex);
    // Activator.getDefault().getLog().log(
    // new Status(Status.ERROR, Activator.PLUGIN_ID, Status.ERROR, meldung, e));
    //			
    // }
  }

  /**
   * @param managementFile
   * @param exportPath
   * @param pids
   * @throws ConfigurationTaskException
   */
  public void startExport(File exportPath, List<String> pids) throws ConfigurationTaskException
  {
    // Alle Dateien sind fr andere gesperrt worden (lock-Dateien). Diese mssen wieder freigegeben werden, sobald
    // die Aktion beendet wurde.
    // starteAutomatischesBeenden();

    if (pids.size() == 1 && pids.get(0).equals(""))
    {
      pids = new ArrayList<String>(((ConfigDataModel)_dataModel).getAllConfigurationAreas().keySet());
    }

    // Export starten
    // try
    {
      ((ConfigDataModel)_dataModel).exportConfigurationAreas(exportPath, pids);
    }
    // catch(Exception ex) {
    // String meldung = "Der Export konnte nicht ordnungsgem durchgefhrt werden";
    // RuntimeException e = new RuntimeException(meldung, ex);
    // Activator.getDefault().getLog().log(
    // new Status(Status.ERROR, Activator.PLUGIN_ID, Status.ERROR, meldung, e));
    // }
  }

  /**
   * Unterzieht alle Bereiche in der aktuellsten Version einer Konsistenzprfung. Die Version muss nicht unbedingt die
   * aktuelle Version sein, sondern die Version, die aktiviert werden knnte.
   * 
   * @param managementFile
   *          Verwaltungsdatei, wird zum erzeugen des DataModel gebraucht
   * @param allowDoublePids
   *          <code>true</code> falls doppelte Pids in verschiedenen Konfigurationsbereichen von der Konsistenzprfung
   *          zugelassen werden sollen.
   * @param ignoreDependencyErrorsInConsistencyCheck
   *          <code>true</code> falls Fehler bei der Prfung der Abhngigkeiten in der Konsistenzprfung ignoriert
   *          werden sollen.
   */
  public void pruefeKonsistenz(final boolean allowDoublePids, final boolean ignoreDependencyErrorsInConsistencyCheck)
      throws Exception
  {
    ((ConfigDataModel)_dataModel).setAllowDoublePids(allowDoublePids);
    ((ConfigDataModel)_dataModel).setIgnoreDependencyErrorsInConsistencyCheck(ignoreDependencyErrorsInConsistencyCheck);
    // Alle Dateien sind fr andere gesperrt worden (lock-Dateien). Diese mssen wieder freigegeben werden, sobald

    final Collection<ConfigurationArea> configurationAreas = ((ConfigDataModel)_dataModel).getAllConfigurationAreas()
        .values();
    // Alle Bereiche mit den jeweiligen Versionen
    final List<ConfigAreaAndVersion> configAreasAndVersions = new ArrayList<ConfigAreaAndVersion>();

    for (ConfigurationArea configurationArea : configurationAreas)
    {
      final short version;
      if (_dataModel.getConfigurationAuthority() == configurationArea.getConfigurationAuthority())
      {
        // In der neusten Version prfen
        version = ((ConfigConfigurationArea)configurationArea).getLastModifiedVersion();
      }
      else
      {
        version = configurationArea.getTransferableVersion();
      }
      configAreasAndVersions.add(new ConfigAreaAndVersion(configurationArea, version));
    }

    final ConsistencyCheckResultInterface consistencyCheckResult = ((ConfigurationControl)_dataModel)
        .checkConsistency(configAreasAndVersions);
    if (consistencyCheckResult != null)
    {
    }

  }

//  /**
//   * Aktiviert alle Bereiche in der hchst mglichen Version.
//   * 
//   * @param managementFile
//   *          Verwaltungsdatei, wird zum erzeugen des DataModel gebraucht
//   * @param allowDoublePids
//   *          <code>true</code> falls doppelte Pids in verschiedenen Konfigurationsbereichen von der Konsistenzprfung
//   *          zugelassen werden sollen.
//   * @param ignoreDependencyErrorsInConsistencyCheck
//   *          <code>true</code> falls Fehler bei der Prfung der Abhngigkeiten in der Konsistenzprfung ignoriert
//   *          werden sollen.
//   */
//  public void starteAktivierung(final boolean allowDoublePids, final boolean ignoreDependencyErrorsInConsistencyCheck)
//  {
//    ((ConfigDataModel)_dataModel).setAllowDoublePids(allowDoublePids);
//    ((ConfigDataModel)_dataModel).setIgnoreDependencyErrorsInConsistencyCheck(ignoreDependencyErrorsInConsistencyCheck);
//
//    // Alle Dateien sind fr andere gesperrt worden (lock-Dateien). Diese mssen wieder freigegeben werden, sobald
//    // die Aktion beendet wurde.
//    // starteAutomatischesBeenden();
//
//    // Alle Bereiche anfordern. Auch die aus der Verwaltungsdatei
//    final Collection<ConfigurationArea> configurationAreas = ((ConfigDataModel)_dataModel).getAllConfigurationAreas()
//        .values();
//
//    final List<ConfigAreaAndVersion> configAreasAndVersions = new ArrayList<ConfigAreaAndVersion>();
//
//    for (ConfigurationArea configurationArea : configurationAreas)
//    {
//      // Den Bereich mit der aktuellsten Version prfen und aktivieren.
//      configAreasAndVersions.add(new ConfigAreaAndVersion(configurationArea));
//    }
//
//    try
//    {
//      ((ConfigurationControl)_dataModel).activateConfigurationAreas(configAreasAndVersions);
//    }
//    catch (Exception ex)
//    {
//      String meldung = "Die Aktivierung konnte nicht ordnungsgem durchgefhrt werden";
//      RuntimeException e = new RuntimeException(meldung, ex);
//      Activator.getDefault().getLog().log(new Status(Status.ERROR, Activator.PLUGIN_ID, Status.ERROR, meldung, e));
//    }
//  }

  /**
   * Aktiviert alle Bereiche in der hchst mglichen Version.
   * 
   * @param managementFile
   *          Verwaltungsdatei, wird zum erzeugen des DataModel gebraucht
   * @param allowDoublePids
   *          <code>true</code> falls doppelte Pids in verschiedenen Konfigurationsbereichen von der Konsistenzprfung
   *          zugelassen werden sollen.
   * @param ignoreDependencyErrorsInConsistencyCheck
   *          <code>true</code> falls Fehler bei der Prfung der Abhngigkeiten in der Konsistenzprfung ignoriert
   *          werden sollen.
   * @throws ConfigurationChangeException
   */
  public void starteAktivierung(Collection<ConfigurationArea> configurationAreas, final boolean allowDoublePids,
      final boolean ignoreDependencyErrorsInConsistencyCheck) throws ConfigurationChangeException
  {
    ((ConfigDataModel)_dataModel).setAllowDoublePids(allowDoublePids);
    ((ConfigDataModel)_dataModel).setIgnoreDependencyErrorsInConsistencyCheck(ignoreDependencyErrorsInConsistencyCheck);

    final List<ConfigAreaAndVersion> configAreasAndVersions = new ArrayList<ConfigAreaAndVersion>();

    for (ConfigurationArea configurationArea : configurationAreas)
    {
      // Den Bereich mit der aktuellsten Version prfen und aktivieren.
      configAreasAndVersions.add(new ConfigAreaAndVersion(configurationArea));
    }

    ((ConfigurationControl)_dataModel).activateConfigurationAreas(configAreasAndVersions);

  }

  // /**
  // * Gibt alle Bereiche zur Aktivierung fr andere Konfigurationen in der hchst mglichen Version frei.
  // *
  // * @param managementFile Verwaltungsdatei, wird zum Erzeugen des DataModel gebraucht
  // */
  // public void starteFreigabeZurAktivierung() {
  // // Alle Dateien sind fr andere gesperrt worden (lock-Dateien). Diese mssen wieder freigegeben werden, sobald
  // // die Aktion beendet wurde.
  // // starteAutomatischesBeenden();
  //
  // // Alle Bereiche anfordern. Die aus der Verwaltungsdatei mssen nicht angefordert werden, weil
  // // die Bereiche noch nicht lokal aktiviert wurden und somit nicht fr andere zur Aktivierung freigegeben
  // // werden knnen.
  // // Es knnen nur die Bereiche zur Aktivierung freigegen werden, fr die die Konfiguration auch verantwortlich ist.
  //
  // // Alle Bereiche, die aktiv sind (mssen noch gecastet werden)
  // final List<SystemObject> areas = _dataModel.getType("typ.konfigurationsBereich").getObjects();
  // // Alle Bereiche, die betrachtet werden mssen. Fr dieses Bereiche ist die Konfiguration auch verantwortlich.
  // final Collection<ConfigurationArea> configurationAreas = new ArrayList<ConfigurationArea>(areas.size());
  // for(SystemObject area : areas) {
  // final ConfigurationArea configurationArea = (ConfigurationArea)area;
  // if(configurationArea.getConfigurationAuthority() == _dataModel.getConfigurationAuthority()) {
  // configurationAreas.add(configurationArea);
  // }
  // }
  //
  // final List<ConfigAreaAndVersion> configAreasAndVersions = new
  // ArrayList<ConfigAreaAndVersion>(configurationAreas.size());
  //
  // for(ConfigurationArea configurationArea : configurationAreas) {
  // // Den Bereich mit der lokal aktivierten Version fr andere freigeben
  // configAreasAndVersions.add(new ConfigAreaAndVersion(configurationArea));
  // }
  //
  // try {
  // ((ConfigurationControl)_dataModel).releaseConfigurationAreasForActivation(configAreasAndVersions);
  // }
  // catch(Exception ex) {
  // String meldung = "Die Freigabe zur Aktivierung konnte nicht ordnungsgem durchgefhrt werden";
  // RuntimeException e = new RuntimeException(meldung, ex);
  // Activator.getDefault().getLog().log(
  // new Status(Status.ERROR, Activator.PLUGIN_ID, Status.ERROR, meldung, e));
  // }
  // }

  /**
   * Gibt alle Bereiche zur Aktivierung fr andere Konfigurationen in der hchst mglichen Version frei.
   * 
   * @param managementFile
   *          Verwaltungsdatei, wird zum Erzeugen des DataModel gebraucht
   * @throws ConfigurationChangeException
   */
  public void starteFreigabeZurAktivierung(Collection<ConfigurationArea> configurationAreas)
      throws ConfigurationChangeException
  {

    final List<ConfigAreaAndVersion> configAreasAndVersions = new ArrayList<ConfigAreaAndVersion>(configurationAreas
        .size());

    for (ConfigurationArea configurationArea : configurationAreas)
    {
      // Den Bereich mit der lokal aktivierten Version fr andere freigeben
      configAreasAndVersions.add(new ConfigAreaAndVersion(configurationArea));      
    }

    // try
    {
      ((ConfigurationControl)_dataModel).releaseConfigurationAreasForActivation(configAreasAndVersions);
    }
    // catch(Exception ex) {
    // String meldung = "Die Freigabe zur Aktivierung konnte nicht ordnungsgem durchgefhrt werden";
    // RuntimeException e = new RuntimeException(meldung, ex);
    // Activator.getDefault().getLog().log(
    // new Status(Status.ERROR, Activator.PLUGIN_ID, Status.ERROR, meldung, e));
    // }
  }

  // /**
  // * Gibt alle Bereiche zur bernahme fr andere Konfigurationen in der hchst mglichen Version frei.
  // *
  // * @param managementFile Verwaltungsdatei, wird zum erzeugen des DataModel gebraucht
  // * @param allowDoublePids <code>true</code> falls doppelte Pids in verschiedenen Konfigurationsbereichen von der
  // Konsistenzprfung zugelassen werden sollen.
  // * @param ignoreDependencyErrorsInConsistencyCheck <code>true</code> falls Fehler bei der Prfung der Abhngigkeiten
  // in der Konsistenzprfung ignoriert
  // * werden sollen.
  // */
  // public void starteFreigabeZurbernahme(final boolean allowDoublePids, final boolean
  // ignoreDependencyErrorsInConsistencyCheck) {
  // ((ConfigDataModel)_dataModel).setAllowDoublePids(allowDoublePids);
  // ((ConfigDataModel)_dataModel).setIgnoreDependencyErrorsInConsistencyCheck(ignoreDependencyErrorsInConsistencyCheck);
  //
  // // Alle Dateien sind fr andere gesperrt worden (lock-Dateien). Diese mssen wieder freigegeben werden, sobald
  // // die Aktion beendet wurde.
  // // starteAutomatischesBeenden();
  //
  // // Alle Bereiche anfordern. Auch die aus der Verwaltungsdatei
  // final Collection<ConfigurationArea> configurationAreas =
  // ((ConfigDataModel)_dataModel).getAllConfigurationAreas().values();
  // final List<ConfigAreaAndVersion> configAreasAndVersions = new ArrayList<ConfigAreaAndVersion>();
  //
  // // Es drfen nur Bereiche freigegeben werden, fr die die Konfiguration auch der KV ist.
  //
  // for(ConfigurationArea configurationArea : configurationAreas) {
  // if(_dataModel.getConfigurationAuthority() == configurationArea.getConfigurationAuthority()) {
  // // Der Bereich wird mit der grt mglichen Version fr andere zur bernahme freigegeben
  // configAreasAndVersions.add(new ConfigAreaAndVersion(configurationArea));
  // }
  // }
  //
  // try {
  // ((ConfigurationControl)_dataModel).releaseConfigurationAreasForTransfer(configAreasAndVersions);
  // }
  // catch(Exception ex) {
  // String meldung = "Die Freigabe zur bernahme konnte nicht ordnungsgem durchgefhrt werden";
  // RuntimeException e = new RuntimeException(meldung, ex);
  // Activator.getDefault().getLog().log(
  // new Status(Status.ERROR, Activator.PLUGIN_ID, Status.ERROR, meldung, e));
  // }
  // }

  /**
   * Gibt alle Bereiche zur bernahme fr andere Konfigurationen in der hchst mglichen Version frei.
   * 
   * @param managementFile
   *          Verwaltungsdatei, wird zum erzeugen des DataModel gebraucht
   * @param allowDoublePids
   *          <code>true</code> falls doppelte Pids in verschiedenen Konfigurationsbereichen von der Konsistenzprfung
   *          zugelassen werden sollen.
   * @param ignoreDependencyErrorsInConsistencyCheck
   *          <code>true</code> falls Fehler bei der Prfung der Abhngigkeiten in der Konsistenzprfung ignoriert
   *          werden sollen.
   * @throws ConfigurationChangeException
   */
  public void starteFreigabeZurUebernahme(Collection<ConfigurationArea> configurationAreas,
      final boolean allowDoublePids, final boolean ignoreDependencyErrorsInConsistencyCheck)
      throws ConfigurationChangeException
  {
    ((ConfigDataModel)_dataModel).setAllowDoublePids(allowDoublePids);
    ((ConfigDataModel)_dataModel).setIgnoreDependencyErrorsInConsistencyCheck(ignoreDependencyErrorsInConsistencyCheck);

    final List<ConfigAreaAndVersion> configAreasAndVersions = new ArrayList<ConfigAreaAndVersion>();

    // Es drfen nur Bereiche freigegeben werden, fr die die Konfiguration auch der KV ist.

    for (ConfigurationArea configurationArea : configurationAreas)
    {
      if (_dataModel.getConfigurationAuthority() == configurationArea.getConfigurationAuthority())
      {
        // Der Bereich wird mit der grt mglichen Version fr andere zur bernahme freigegeben
        configAreasAndVersions.add(new ConfigAreaAndVersion(configurationArea));
      }
    }

    // try
    {
      ((ConfigurationControl)_dataModel).releaseConfigurationAreasForTransfer(configAreasAndVersions);
    }
    // catch(Exception ex) {
    // String meldung = "Die Freigabe zur bernahme konnte nicht ordnungsgem durchgefhrt werden";
    // RuntimeException e = new RuntimeException(meldung, ex);
    // Activator.getDefault().getLog().log(
    // new Status(Status.ERROR, Activator.PLUGIN_ID, Status.ERROR, meldung, e));
    // }
  }

  /**
   * Gibt die angegebenen Bereiche zur Aktivierung fr andere Konfigurationen in der hchst mglichen Version frei. Die
   * entsprechenden Bereiche mssen vorher nicht lokal aktiviert worden sein.
   * 
   * @param managementFile
   *          Verwaltungsdatei, wird zum erzeugen des DataModel gebraucht
   * @param pids
   *          Pids der Konfigurationsbereiche, die freigegeben werden sollen.
   * @param allowDoublePids
   *          <code>true</code> falls doppelte Pids in verschiedenen Konfigurationsbereichen von der Konsistenzprfung
   *          zugelassen werden sollen.
   * @param ignoreDependencyErrorsInConsistencyCheck
   *          <code>true</code> falls Fehler bei der Prfung der Abhngigkeiten in der Konsistenzprfung ignoriert
   *          werden sollen.
   * @throws Exception
   */
  public void startFreigabeZurAktivierungOhneKVAktivierung(Collection<ConfigurationArea> configurationAreas,
      final boolean allowDoublePids, final boolean ignoreDependencyErrorsInConsistencyCheck) throws Exception
  {
    ((ConfigDataModel)_dataModel).setAllowDoublePids(allowDoublePids);
    ((ConfigDataModel)_dataModel).setIgnoreDependencyErrorsInConsistencyCheck(ignoreDependencyErrorsInConsistencyCheck);

    final List<ConfigAreaAndVersion> configAreasAndVersions = new ArrayList<ConfigAreaAndVersion>();

    // Es drfen nur Bereiche freigegeben werden, fr die die Konfiguration auch der KV ist.

    for (ConfigurationArea configurationArea : configurationAreas)
    {
      if (_dataModel.getConfigurationAuthority() == configurationArea.getConfigurationAuthority())
      {
        // Der Bereich wird mit der grt mglichen Version fr andere zur bernahme freigegeben
        configAreasAndVersions.add(new ConfigAreaAndVersion(configurationArea));
      }
    }

    // try
    {
      final ConsistencyCheckResultInterface consistencyCheckResult = ((ConfigurationControl)_dataModel)
          .releaseConfigurationAreasForActivationWithoutCAActivation(configAreasAndVersions);

      if (consistencyCheckResult.interferenceErrors())
      {
        // Es gab Interfernezfehler. Die Bereiche knnen nicht fr anderen freigegeben werden.
        String meldung = "Bei der Aktivierung durch andere ohne vorherige Aktivierung durch den KV wurden folgende Inkonsistenzen erkannt, die nicht zu einem Abbruch der Aktion gefhrt haben: "
            + consistencyCheckResult;
        RuntimeException e = new RuntimeException(meldung);

        throw new Exception(meldung);

        // Activator.getDefault().getLog().log(
        // new Status(Status.WARNING, Activator.PLUGIN_ID, Status.WARNING, meldung, e));
        //			  
        // _debug.warning(
        // "Bei der Aktivierung durch andere ohne vorherige Aktivierung durch den KV wurden folgende Inkonsistenzen erkannt, die nicht zu einem Abbruch der Aktion gefhrt haben: "
        // + consistencyCheckResult
        // );
      }
    }
    // catch(Exception ex) {
    // String meldung =
    // "Die Freigabe zur Aktivierung ohne Aktivierung durch den KV konnte nicht ordnungsgem durchgefhrt werden";
    // RuntimeException e = new RuntimeException(meldung, ex);
    // Activator.getDefault().getLog().log(
    // new Status(Status.ERROR, Activator.PLUGIN_ID, Status.ERROR, meldung, e));
    // }
  }

  /**
   * Legt einen Thread an, der ausgefhrt wird, wenn das System beendet wird. Dieser Thread wird alle Daten der
   * Konfiguration sichern.
   */
  public void starteAutomatischesBeenden()
  {
    if (_autoCloser == null)
    {
      _autoCloser = new AutomatischesFreigeben(_dataModel);
      Runtime.getRuntime().addShutdownHook(new Thread(_autoCloser));
    }
  }

  // /** Runnable Implementierung, die beim Beenden und zyklisch dafr sorgt, dass ungesicherte Konfigurationsnderungen
  // gespeichert werden. */
  // private class AutoSicherung extends TimerTask implements Runnable {
  //
  // private final DataModel _dataModel;
  //
  // public AutoSicherung(DataModel dataModel) {
  // _dataModel = dataModel;
  // }
  //
  // /** Methode, die einerseits durch den Timer zyklisch aufgerufen wird und andererseits beim Beenden der
  // Konfiguration. */
  // public void run() {
  // try {
  // ((ConfigDataModel)_dataModel).save();
  // }
  // catch(IOException ex) {
  // // falls das Speichern nicht gelingt wird nur eine Warnung ausgegeben, evtl. gelingt es beim nchsten Mal
  // String meldung = "Die Verwaltungsdaten und die Konfigurationsdateien konnten nicht gespeichert werden";
  // RuntimeException e = new RuntimeException(meldung, ex);
  // Aktivator.getDefault().getLog().log(
  // new Status(Status.WARNING, Aktivator.PLUGIN_ID, Status.OK, meldung, e));
  //			  
  // }
  // }
  // }

  /**
   * Dieser Thread wird aufgerufen, wenn die Konfiguration freigegeben werden soll. Es wird am ConfigDataModel
   * <code>close</code> aufgerufen.
   */
  private class AutomatischesFreigeben extends TimerTask implements Runnable
  {

    private final DataModel _dataModel;

    // private final ConfigurationCommunicator _configurationCommunicator;

    /**
     * Objekte, an denen die Close Methode aufgerufen wird.
     * 
     * @param dataModel
     *          Datenmodell, an dem close aufgerufen wird. <code>null</code> darf nicht bergeben werden.
     * @param configurationCommunicator
     *          Objekt, an dem close aufgerufen wird. Soll an diesem Objekte kein close aufgerufen werden, muss
     *          <code>null</code> aufgerufen werden.
     */
    public AutomatischesFreigeben(DataModel dataModel)
    {
      _dataModel = dataModel;

    }

    public void run()
    {

      ((ConfigDataModel)_dataModel).close();

    }
  }
}
