/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.datenbereich.datenbasis;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDynamicObject;

/**
 * Diese Klasse liefert die Basis fr GTM-artige Darstellungen der Daten des DaV. Es knnen entweder alle Objekte des
 * jeweiligen DaV dargestellt werden (durch einen Aufruf von {@link GtmDatenBasisOffline#setAll()}) oder Systemobjekte von
 * bestimmten Objekttypen ({@link GtmDatenBasisOffline#setTypes(List)}). Es sind Getter fr SystemObjekte, SystemObjektTypen,
 * Attributgruppen und Aspekte vorhanden. Die Methode {@link GtmDatenBasisOffline#filtere(List, List, List, List)} ist in der
 * Lage, SystemObjekte nach SystemObjektTyp, Attributgruppe und Aspekt zu filtern.
 * 
 * @author HofmannJ
 * 
 */
public class GtmDatenBasisOffline implements Cloneable
{
  /**
   * Verbindung zum DaV
   */
//  private ClientDavInterface _connection = null;

  /**
   * Datenmodell des DaV
   */
  private DataModel _dataModel = null;

  /**
   * Nimmt die SystemObjekte auf.
   */
  private TreeMap<String, GtmListenElementObjekt> _tmSystemObjects = new TreeMap<String, GtmListenElementObjekt>();

  /**
   * Nimmt die SystemObjektTypen auf
   */
  private TreeMap<String, SystemObjectType> _tmSystemObjectTypes = new TreeMap<String, SystemObjectType>();

  /**
   * Nimmt die Attributgruppen auf
   */
  private TreeMap<String, AttributeGroup> _tmAtg = new TreeMap<String, AttributeGroup>();

  /**
   * Nimmt die Aspekte auf
   */
  private TreeMap<String, Aspect> _tmAsp = new TreeMap<String, Aspect>();

  /**
   * Nimmt die Attributgruppen auf
   */
  private TreeMap<String, AttributeGroup> _tmAtgAlle = new TreeMap<String, AttributeGroup>();

  /**
   * Nimmt die Aspekte auf
   */
  private TreeMap<String, Aspect> _tmAspAlle = new TreeMap<String, Aspect>();

  /**
   * Monitor
   */
  private IProgressMonitor _monitor = null;

  /**
   * Maximale Anzahl der Versuche zur Bestimmungen der zu einem SystemObjektTyp zugehrigen SystemObjekte. Erst wenn
   * diese Anzahl erreicht ist, wird eine Meldung im Eclipse-Log erzeugt.
   */
  private static final int _maxAnzVersuche = 3;

  /**
   * Konstruktor
   * 
   * @param connection
   *          Verbindung zum DaV
   * @param selfInit
   *          Wenn <code>true</code>, wird die Struktur aus allen im DaV verfgbaren Informationen aufgebaut, bei
   *          <code>false</code> nicht.
   * @param monitor
   *          Hier kann der {@link IProgressMonitor} des {@link Job}s bergeben werden. In diesem wird dann der aktuelle
   *          Konfigurationsbereich ausgegeben, dessen SystemObjekte gerade geladen werden. Wenn dieser Parameter
   *          <code>null</code> ist, so wird diese Ausgabe unterdrckt.
   */
  public GtmDatenBasisOffline(DataModel dataModel, boolean selfInit, IProgressMonitor monitor)
  {
      
    this._dataModel = dataModel;

    this._monitor = monitor;

    if (selfInit)
    {
      setAll();
    }
  }

  /**
   * Fllt die Struktur mit allen vom DaV gelieferten {@link SystemObject}s und {@link SystemObjectType}s.
   */
  public void setAll()
  {
    _tmSystemObjects.clear();
    _tmSystemObjectTypes.clear();

    Collection<SystemObject> systemObjects = null;
    SystemObjectType sot = _dataModel.getType("typ.konfigurationsBereich");
    List<SystemObject> listeSOTs = sot.getElements();
    Iterator<SystemObject> itListeSOTs = listeSOTs.iterator();
    while (itListeSOTs.hasNext())
    {
      SystemObject objektSOT = itListeSOTs.next();
     
      LinkedList<SystemObject> sysobjs = bestimmeSystemObjekteZuSOT(objektSOT, 0);
      if (sysobjs != null)
      {
        if (sysobjs.size() > 0)
        {
          if (systemObjects == null)
          {
            systemObjects = new LinkedList<SystemObject>();
          }

          systemObjects.addAll(sysobjs);
        }
      }

    }
 
    if (systemObjects != null)
    {
      Iterator<SystemObject> itSO = systemObjects.iterator();
      while (itSO.hasNext())
      {
        SystemObject systemObject = itSO.next();

        if (systemObject != null)
        {

          GtmListenElementObjekt leo = convertSystemObjectToListenElement(systemObject);

          if (!_tmSystemObjects.containsKey(leo.getName()))
          {
            _tmSystemObjects.put(leo.getName(), leo);

            SystemObjectType type = systemObject.getType();
            if (!_tmSystemObjectTypes.containsKey(type.getPid()))
            {
              _tmSystemObjectTypes.put(type.getPid(), type);
            }
          }
        }
      }
    }
    initAlle();
  }

  /**
   * Bestimmt zu den Konfigurationsbereich eines SystemObjektTyps alle zugehrigen SystemObjekte.
   * 
   * @param sot
   *          SystemObjektTyp, von dessen Konfigurationsbereich alle aktuell gltigen SystemObjekte bestimmt werden
   *          sollen.
   * @param versuch
   *          Gibt die Nummer des Versuchs an. Wird zu Bestimmung der Tiefe der rekursiven Aufrufe bentigt. Ein guter
   *          initialer Wert fr den ersten Durchlauf ist 0.
   * @return Liefert eine Liste der bestimmten SystemObjekte zurck.
   */
  private LinkedList<SystemObject> bestimmeSystemObjekteZuSOT(SystemObject sot, int versuch)
  {

    LinkedList<SystemObject> re = null;

    ConfigurationArea ca = sot.getConfigurationArea();

    if (_monitor != null)
    {
    	String msg = "Lese Konfigurationsbereich \"" + ca.getPid() + "\", Leseversuch: " + versuch;
      _monitor.setTaskName(msg);
    }

    try
    {
      final Collection<SystemObject> tmp = ca.getObjects(null, ObjectTimeSpecification.valid());
      
      List<SystemObject> sysobjs = new ArrayList<SystemObject>();
      
      for (SystemObject so : tmp)
      {
        if (so instanceof ConfigurationObject)
        {
          ConfigurationObject ko = (ConfigurationObject)so;
          if (!(ko.getNotValidSince() >= ko.getConfigurationArea().getModifiableVersion()))
          {
            // Objekt wird in zuknftiger Version nicht ungltig
            sysobjs.add(so);
            
          }
//        else
//        {
//          System.out.println("Objekt ist in der nchsten Version ungltig: " + so.getPid());
//        }        
        }else if (so instanceof DynamicObject){
          DynamicObject dyo = (DynamicObject)so;
          if (!(dyo.getNotValidSince() >= dyo.getConfigurationArea().getModifiableVersion()))
          {
            // Objekt wird in zuknftiger Version nicht ungltig
            sysobjs.add(so);
            
          }
        }
      }
      
      Collection<SystemObject> newObjects = ca.getNewObjects();
      if (newObjects != null){ 
        sysobjs.addAll(newObjects); 
      }

      if (sysobjs.size() > 0)
      {
        if (re == null)
        {
          re = new LinkedList<SystemObject>();
        }
        re.addAll(sysobjs);
      }
    }
    catch (RuntimeException runex)
    {
      String msg = "GtmDatenBasis::bestimmeSystemObjekteZuSOT => Gescheitert in Versuch " + versuch
          + ", Konfigurationsbereich: " + ca.getPid();

      if (versuch == _maxAnzVersuche)
      {
        Activator.getDefault().getLog().log(new Status(Status.WARNING, Activator.PLUGIN_ID, msg, runex));
      }

      versuch++;
      if (versuch <= _maxAnzVersuche)
      {
        re = bestimmeSystemObjekteZuSOT(sot, versuch);
      }
    }

    return re;
  }

  /**
   * Fllt die Struktur mit den bergebenen {@link SystemObjectType}s und sucht dazu vom DaV die zugehrigen
   * {@link SystemObject}s heraus.
   * 
   * @param types
   *          {@link SystemObjectType}s.
   */
  public void setTypes(List<SystemObjectType> types)
  {
    _tmSystemObjects.clear();
    _tmSystemObjectTypes.clear();

    Iterator<SystemObjectType> itTypes = types.iterator();
    {
      while (itTypes.hasNext())
      {
        SystemObjectType type = itTypes.next();
        _tmSystemObjectTypes.put(type.getPid(), type);
      }
    }
    
    Collection<SystemObject> tmp = _dataModel.getObjects(null, types, ObjectTimeSpecification.valid());

    List<SystemObject> sysobjs = new ArrayList<SystemObject>();
    
    for (SystemObject so : tmp)
    {
      if (so instanceof ConfigurationObject)
      {
        ConfigurationObject ko = (ConfigurationObject)so;
        if (!(ko.getNotValidSince() >= ko.getConfigurationArea().getModifiableVersion()))
        {
          // Objekt wird in zuknftiger Version nicht ungltig
          sysobjs.add(so);
          
        }
//      else
//      {
//        System.out.println("Objekt ist in der nchsten Version ungltig: " + so.getPid());
//      }        
      }else if (so instanceof DynamicObject){
        DynamicObject dyo = (DynamicObject)so;
        if (!(dyo.getNotValidSince() >= dyo.getConfigurationArea().getModifiableVersion()))
        {
          // Objekt wird in zuknftiger Version nicht ungltig
          sysobjs.add(so);
          
        }
      }             
    }
    
    
    // Noch nicht aktivierte Objekte werden  auch der Auswahl hinzugefgt
    ConfigDataModel config = (ConfigDataModel)_dataModel;
    
    Map<String, ConfigurationArea> areas = config.getAllConfigurationAreas();
    
    for (Map.Entry<String, ConfigurationArea> me : areas.entrySet())
    {
      ConfigurationArea ca = me.getValue();
      
      Collection<SystemObject> newObjects = ca.getNewObjects();
      
      for (SystemObject so : newObjects)
      {
        for (SystemObjectType type : types)
        {
          if (so.isOfType(type)){            
            sysobjs.add(so); 
          }          
        }
      }    
    }
        
    Iterator<SystemObject> itSO = sysobjs.iterator();
    while (itSO.hasNext())
    {
      SystemObject systemObject = itSO.next();
     
      GtmListenElementObjekt leo = convertSystemObjectToListenElement(systemObject);
      _tmSystemObjects.put(leo.getName(), leo);
    }

    initAlle();
  }

  /**
   * Fllt die Struktur mit den bergebenen {@link SystemObject}s und bestimmt die zugehrigen {@link SystemObjectType}
   * s.
   * 
   * @param systemObjects
   *          {@link SystemObject}s.
   */
  public void setSystemObjects(List<SystemObject> systemObjects)
  {
    _tmSystemObjects.clear();
    _tmSystemObjectTypes.clear();

    Iterator<SystemObject> itSO = systemObjects.iterator();
    while (itSO.hasNext())
    {
      SystemObject systemObject = itSO.next();
      GtmListenElementObjekt leo = convertSystemObjectToListenElement(systemObject);

      // _tmSystemObjects.put(systemObject.getPid(), systemObject);
      _tmSystemObjects.put(leo.getName(), leo);

      SystemObjectType type = systemObject.getType();
      if (!_tmSystemObjectTypes.containsKey(type.getPid()))
      {
        _tmSystemObjectTypes.put(type.getPid(), type);
      }
    }

    initAlle();
  }

  public void initAlle()
  {
    Iterator<AttributeGroup> itAtg = getAtg(getSystemObjectTypes()).iterator();
    while (itAtg.hasNext())
    {
      AttributeGroup attributeGroup = itAtg.next();
      _tmAtgAlle.put(attributeGroup.getPid(), attributeGroup);
    }

    Iterator<Aspect> itAsp = getAsp(getAlleAtg()).iterator();
    while (itAsp.hasNext())
    {
      Aspect aspect = itAsp.next();
      _tmAspAlle.put(aspect.getPid(), aspect);
    }
  }

  /**
   * Liefert eine Liste von {@link SystemObject}s, passend zu den durch die Setter getroffene Auswahl zurck.
   * 
   * @return Liste von {@link SystemObject}s
   */
  // public List<SystemObject> getSystemObjects()
  public List<GtmListenElementObjekt> getSystemObjects()
  {
    // LinkedList<SystemObject> re = new LinkedList<SystemObject>();
    LinkedList<GtmListenElementObjekt> re = new LinkedList<GtmListenElementObjekt>();

    // Iterator<Entry<String, SystemObject>> it = _tmSystemObjects.entrySet()
    // .iterator();
    Iterator<Entry<String, GtmListenElementObjekt>> it = _tmSystemObjects.entrySet().iterator();
    while (it.hasNext())
    {
      // SystemObject systemObject = it.next().getValue();
      GtmListenElementObjekt leo = it.next().getValue();
      // re.add(systemObject);
      re.add(leo);
    }

    return re;
  }

  /**
   * Liefert eine Liste von {@link SystemObjectType}s, passend zu die durch die Setter getroffene Auswahl zurck.
   * 
   * @return
   */
  public List<SystemObjectType> getSystemObjectTypes()
  {
    LinkedList<SystemObjectType> re = new LinkedList<SystemObjectType>();

    Iterator<Entry<String, SystemObjectType>> it = _tmSystemObjectTypes.entrySet().iterator();
    while (it.hasNext())
    {
      SystemObjectType systemObjectType = it.next().getValue();
      re.add(systemObjectType);
    }

    return re;
  }

  /**
   * Bestimmt eine Liste von {@link AttributeGroup}s passend zu einer Liste von {@link SystemObjectType}s.
   * 
   * @param listSystemObjectTypes
   *          Liste von {@link SystemObjectType}s.
   * @return Liste von {@link AttributeGroup}s
   */
  public List<AttributeGroup> getAtg(List<SystemObjectType> listSystemObjectTypes)
  {
    LinkedList<AttributeGroup> re = new LinkedList<AttributeGroup>();

    _tmAtg.clear();
    Iterator<SystemObjectType> itSOT = listSystemObjectTypes.iterator();
    while (itSOT.hasNext())
    {
      SystemObjectType systemObjectType = itSOT.next();
      Iterator<AttributeGroup> itAtg = systemObjectType.getAttributeGroups().iterator();
      while (itAtg.hasNext())
      {
        AttributeGroup attributeGroup = itAtg.next();
        if (!_tmAtg.containsKey(attributeGroup.getPid()))
        {
          _tmAtg.put(attributeGroup.getPid(), attributeGroup);
        }
      }
    }

    Iterator<Entry<String, AttributeGroup>> it = _tmAtg.entrySet().iterator();
    while (it.hasNext())
    {
      AttributeGroup attributeGroup = it.next().getValue();
      re.add(attributeGroup);
    }

    return re;
  }

  /**
   * Bestimmt zu einer Liste von {@link AttributeGroup}s eine Liste von {@link Aspect}s.
   * 
   * @param listAttributeGroups
   *          Liste von {@link AttributeGroup}s
   * @return Liste von {@link Aspect}s.
   */
  public List<Aspect> getAsp(List<AttributeGroup> listAttributeGroups)
  {
    _tmAsp.clear();

    LinkedList<Aspect> re = new LinkedList<Aspect>();

    Iterator<AttributeGroup> itAtg = listAttributeGroups.iterator();
    while (itAtg.hasNext())
    {
      AttributeGroup attributeGroup = itAtg.next();
      Iterator<Aspect> itAsp = attributeGroup.getAspects().iterator();
      while (itAsp.hasNext())
      {
        Aspect aspect = itAsp.next();
        if (!_tmAsp.containsKey(aspect.getPid()))
        {
          _tmAsp.put(aspect.getPid(), aspect);
        }
      }
    }

    Iterator<Entry<String, Aspect>> it = _tmAsp.entrySet().iterator();
    while (it.hasNext())
    {
      Aspect aspect = it.next().getValue();
      re.add(aspect);
    }

    return re;
  }

  public LinkedList<AttributeGroup> getAlleAtg()
  {
    LinkedList<AttributeGroup> re = new LinkedList<AttributeGroup>();

    Iterator<Entry<String, AttributeGroup>> it = _tmAtgAlle.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<String, AttributeGroup> entry = it.next();
      re.add(entry.getValue());
    }

    return re;
  }

  public LinkedList<Aspect> getAlleAsp()
  {
    LinkedList<Aspect> re = new LinkedList<Aspect>();

    Iterator<Entry<String, Aspect>> it = _tmAspAlle.entrySet().iterator();
    while (it.hasNext())
    {
      Entry<String, Aspect> entry = it.next();
      re.add(entry.getValue());
    }

    return re;
  }

  /**
   * Filtert eine Liste von SystemObjekten (verpackt als {@link GtmListenElementObjekt}e) an Hand von SystemObjekttypen,
   * Attributgruppen sowie Aspekten.
   * 
   * @param sos
   *          Liste von {@link GtmListenElementObjekt}en. Zwingend notwendig.
   * @param types
   *          Liste von {@link SystemObjectType}s, gegen welche die {@link SystemObject}s gefiltert werden sollen oder
   *          <code>null</code>, wenn nicht gegen {@link SystemObjectType}s gefiltert werden soll.
   * @param atgs
   *          Liste von {@link AttributeGroup}s, gegen welche die {@link SystemObject}s gefiltert werden sollen oder
   *          <code>null</code>, wenn nicht gegen die {@link AttributeGroup}s gefiltert werden soll.
   * @param asps
   *          Liste von {@link Aspect}s, gegen welche die {@link SystemObject}s gefiltert werden sollen oder
   *          <code>null</code>, wenn nicht gegen die {@link Aspect}s gefiltert werden soll.
   * @return Gefilterte Liste von {@link SystemObject}s. Waren die Filtereinstellungen fr {@link SystemObjectType}s,
   *         {@link AttributeGroup}s und {@link Aspect}s jeweils null, so wird als Parameter vorgegebene Liste von
   *         {@link SystemObject} (verpackt als {@link GtmListenElementObjekt}) zurckgeliefert.
   */
  public LinkedList<GtmListenElementObjekt> filtere(List<GtmListenElementObjekt> sos, List<SystemObjectType> types,
      List<AttributeGroup> atgs, List<Aspect> asps)
  {
    LinkedList<GtmListenElementObjekt> re = new LinkedList<GtmListenElementObjekt>();

    if ((sos != null) && (sos.size() > 0))
    {
      Iterator<GtmListenElementObjekt> itSystemObject = sos.iterator();
      while (itSystemObject.hasNext())
      {
        GtmListenElementObjekt leo = itSystemObject.next();
        SystemObject systemObject = leo.getSystemObjekt();

        /*
         * Type-Prfung
         */
        boolean typOk = false;
        if (types != null)
        {
          if (types.contains(systemObject.getType()))
          {
            typOk = true;
          }
        }
        else
        {
          typOk = true;
        }

        /*
         * AttributGruppe
         */
        boolean atgOk = true;
        if (typOk)
        {
          if (atgs != null)
          {
            List<AttributeGroup> atgsSo = systemObject.getType().getAttributeGroups();

            Iterator<AttributeGroup> itAtg = atgs.iterator();
            while (itAtg.hasNext())
            {
              AttributeGroup attributeGroup = itAtg.next();
              if (!atgsSo.contains(attributeGroup))
              {
                atgOk = false;
                break;
              }
            }
          }
        }

        /*
         * Aspekt
         */
        boolean aspOk = true;

        if (typOk && atgOk)
        {
          if (asps != null)
          {
            /*
             * Keine Attributgruppen bergeben. Es werden alle Atgs des jeweiligen Typs durchsucht!
             */
            if (atgs == null)
            {
              LinkedList<Aspect> aspectsSOIst = new LinkedList<Aspect>();

              Iterator<AttributeGroup> itAtgSo = systemObject.getType().getAttributeGroups().iterator();
              while (itAtgSo.hasNext())
              {
                AttributeGroup attributeGroupSo = itAtgSo.next();
                Iterator<Aspect> itAspSo = attributeGroupSo.getAspects().iterator();
                while (itAspSo.hasNext())
                {
                  Aspect aspSo = itAspSo.next();
                  if (!aspectsSOIst.contains(aspSo))
                  {
                    aspectsSOIst.add(aspSo);
                  }
                }
              }

              aspOk = sindAlleElementeInListe(asps, aspectsSOIst);
            }
            else
            {
              /*
               * Attributgruppen bergben.
               */
              LinkedList<Aspect> aspectsSOIst = new LinkedList<Aspect>();
              Iterator<AttributeGroup> itAtg = atgs.iterator();
              while (itAtg.hasNext())
              {
                AttributeGroup attributeGroup = itAtg.next();
                Iterator<Aspect> itAsp = attributeGroup.getAspects().iterator();
                while (itAsp.hasNext())
                {
                  Aspect aspect = itAsp.next();
                  if (!aspectsSOIst.contains(aspect))
                  {
                    aspectsSOIst.add(aspect);
                  }
                }
              }

              aspOk = sindAlleElementeInListe(asps, aspectsSOIst);
            }
          }
        }

        /*
         * Zur Liste hinzufgen
         */
        if (typOk && atgOk && aspOk)
        {
          re.add(leo);
        }

      }
    }
    else
    {
      Iterator<Entry<String, GtmListenElementObjekt>> it = _tmSystemObjects.entrySet().iterator();
      while (it.hasNext())
      {
        re.add(it.next().getValue());
      }
    }

    return re;
  }

  /**
   * Prft, ob die Elemente aus einer Muss-Liste in einer Ist-Liste vorhanden sind.
   * 
   * @param listMuss
   *          Muss-Liste
   * @param listIst
   *          Ist-Liste
   * @return Liefert <code>true</code>, wenn alle Elemente aus der Muss-Liste in der Ist-Liste vorhanden sind.
   */
  private boolean sindAlleElementeInListe(List<Aspect> listMuss, List<Aspect> listIst)
  {
    boolean re = true;

    Iterator<Aspect> it = listMuss.iterator();
    while (it.hasNext())
    {
      Object aspMuss = it.next();
      if (!listIst.contains(aspMuss))
      {
        re = false;
        break;
      }
    }

    return re;
  }

  /**
   * Konvertiert eine Liste von SystemObjekten in eine Liste von {@link GtmListenElementObjekt}en.
   * 
   * @param systemObjekte
   *          Zu konvertierende SystemObjekte.
   * @return Erstellte Liste von {@link GtmListenElementObjekt}en.
   */
  protected static LinkedList<GtmListenElementObjekt> convertSystemObjectsToListenElement(
      List<SystemObject> systemObjekte)
  {
    // LinkedList<GtmListenElementObjekt> re = null;
    LinkedList<GtmListenElementObjekt> re = new LinkedList<GtmListenElementObjekt>();

    Iterator<SystemObject> it = systemObjekte.iterator();
    while (it.hasNext())
    {
      SystemObject so = it.next();
      if (so != null)
      {
        GtmListenElementObjekt leo = new GtmListenElementObjekt(so);
        re.add(leo);
      }
    }

    return re;
  }

  /**
   * Konvertiert eine SystemObjekt in ein {@link GtmListenElementObjekt}
   * 
   * @param systemObject
   *          Zu konvertierendes {@link SystemObject}
   * @return Erstelltes {@link GtmListenElementObjekt}
   */
  protected static GtmListenElementObjekt convertSystemObjectToListenElement(SystemObject systemObject)
  {
    GtmListenElementObjekt re = null;

    if (systemObject != null)
    {
      re = new GtmListenElementObjekt(systemObject);
    }

    return re;
  }

  public Object clone()
  {
    GtmDatenBasisOffline kopie = new GtmDatenBasisOffline(_dataModel, false, _monitor);
    kopie._dataModel = this._dataModel;

    kopie._tmSystemObjects.putAll(this._tmSystemObjects);
    kopie._tmSystemObjectTypes.putAll(this._tmSystemObjectTypes);
    kopie._tmAtg.putAll(this._tmAtg);
    kopie._tmAsp.putAll(this._tmAsp);

    return kopie;
  }

  public GtmDatenBasisOffline flacheKopie()
  {
    GtmDatenBasisOffline re = null;
    re = (GtmDatenBasisOffline)this.clone();
    return re;
  }

}
