/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.pruefungen;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieManager;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.AttributeListDefinition;
import de.bsvrz.dav.daf.main.config.AttributeType;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.ReferenceAttributeType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;

/**
 * Methoden zum Ermitteln, welche konkreten Systemobjekte gelscht werden und ob und welche offenen Referenzen enstehen,
 * wenn ein oder mehrere Hierarchieobjekte gelscht werden sollen.
 * 
 * @author Sans
 * 
 */
public class LoeschenChecker
{
  /**
   * Die Hierarchie, in der sich die zu lschende Hierarchieobjekte befinden
   */
  private IHierarchie _hierarchie;

  /**
   * Der HierarchieManager, in der sich die zu lschende Hierarchieobjekte befinden
   */
  private IHierarchieManager _hierarchieManager;

  /**
   * Menge der Attributgruppen, die ein referenzierendes Attribut enthalten
   */
  private Set<AttributeGroup> _atgMitReferenz;

  /**
   * Liste von Pids aller Typen und Mengen des Meta-Modells
   */
  private final static String[] PIDS_META_MODELL_TYPEN = new String[] { "typ.typ", "typ.attributListenDefinition",

  "typ.attributTyp", "typ.attributgruppe", "typ.attribut", "typ.attributgruppenVerwendung", "typ.werteZustand",
      "typ.mengenVerwendung", "typ.werteBereich", "typ.aspekt", "menge.attribute", "menge.mengenVerwendungen",
      "menge.attributgruppenVerwendungen", "menge.objektTypen", "menge.attributgruppen", "menge.werteZustaende" };

  /**
   * Map aller in einem Konfigurationsbereich vorkommenden Metamodell-Typen
   */
  Map<ConfigurationArea, Set<SystemObjectType>> _metaModellTypen = new HashMap<ConfigurationArea, Set<SystemObjectType>>();


  /**
   * Es wird eine neue Instanz vom Typ <code>LoeschenChecker</code> erzeugt.
   * 
   * @param hierarchieManager
   *          der Hierarchiemanager
   */
  public LoeschenChecker(IHierarchieManager hierarchieManager)
  {
    _hierarchieManager = hierarchieManager;
    _hierarchie = hierarchieManager.getHierarchie();
    initialisiereMetaModellTypen();
    _atgMitReferenz = bestimmeAtgMitReferenz();
  }

  /**
   * Fhrt die Prfungen fr das Lschen eines oder mehrerer Hierarchieobjekte durch
   * 
   * @param hierarchieObjekte
   *          Feld der zu lschenden Hierarchieobjekte
   * @return Eine Statusmeldung, die die zu lschenden konkreten Systemobjkete und die ggf. die durch das Lschen
   *         enstehenden offenen Referenzen ausgibt. Die Statusmeldung hat den Grad WARNING, wenn offene Referenzen
   *         gefunden wurden.
   */
  public IStatus pruefe(HierarchieObjekt[] hierarchieObjekte)
  {
    Set<SystemObject> konkreteObjekte = new LinkedHashSet<SystemObject>();

    String meldung = "Prfen Lschen " + hierarchieObjekte.length + " Hierarchieobjekt(e): ";
    for (int i = 0; i < hierarchieObjekte.length; i++)
    {
      meldung += hierarchieObjekte[i];
      if (i < hierarchieObjekte.length - 1)
      {
        meldung += ", ";
      }
    }
    MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK, meldung, null);
    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
        "Durch das Lschen werden folgende konkrete Objekte gelscht:", null);

    try
    {
      boolean flag = _hierarchieManager.verwendeZuLoeschendeObjekteZurPruefungOffeneReferenzen();
      Collection<SystemObjectType> typen = new ArrayList<SystemObjectType>();
      Collection<SystemObject> ausschlussListe = flag ? null : new ArrayList<SystemObject>();
      

      for (HierarchieObjekt ho : hierarchieObjekte)
      {
        bestimmeKonkreteObjekte(ho, konkreteObjekte, status);

        bestimmeVerboteneTypen(ho, typen);
        
        // nderung 20100916: Es wird angenommen, dass ein direktes Eltern-Objekt zu einem Hierarchieobjekt
        // nie geprft werden muss, da eventuell entstehende offene Referenzen beim Lschen in den TKAs stets
        // korrekt behandelt werden
        if (!flag)
        {
          Object vater = _hierarchie.getUebergeordnetesObjekt(ho);
          if (vater instanceof HierarchieObjekt)
          {
            for (SystemObject so: ((HierarchieObjekt)vater).getSystemObjekte())
            {
              ausschlussListe.add(so);
            }
          }
        }
      }
      result.add(status);

//      if (liste != null && liste.size() > 0)
//      {
//        boolean flag = _hierarchieManager.verwendeZuLoeschendeObjekteZurPruefungOffeneReferenzen();
//        result.add(bestimmeOffeneReferenzen(konkreteObjekte, liste, flag));
//      }
//      else
//      {
//        result.add(bestimmeOffeneReferenzen(konkreteObjekte));
//      }
      
      
      result.add(bestimmeOffeneReferenzen(konkreteObjekte, typen, ausschlussListe, flag));
    }
    catch (RuntimeException e)
    {
      result.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Laufzeit-Ausnahme beim Prfen", e));
    }

    return result;
  }

  /**
   * Bestimmt die konkreten Systemobjekte, die gelscht werden, wenn ein Hierarchieobjekt gelscht werden soll, und fgt
   * sie an eine Liste an. Wenn eine Multimeldung bergeben wird, wird diese um eine entsprechende Statusmeldung ergnzt
   * 
   * @param ho
   *          das zu lschende Hierarchieobjekt
   * @param konkreteObjekte
   *          die Liste konkreter Systemobjekte, an die die bestimmten Objekte angehngt werden
   * @param meldung
   *          die Multimeldung, in die fr jedes gefundene konkrete Objekt eine Statusmeldung eingefgt wird
   */
  private void bestimmeKonkreteObjekte(HierarchieObjekt ho, Collection<SystemObject> konkreteObjekte,
      MultiStatus meldung)
  {
    if (ho.getHierarchieObjektTyp().isSystemObjektVorhanden())
    {
      return;
    }

    for (SystemObject so : ho.getSystemObjekte())
    {
      konkreteObjekte.add(so);
      if (meldung != null)
      {  
        meldung.add(new Status(Status.OK, Activator.PLUGIN_ID, "Objekttyp " + so.getType().getPid() + " mit Pid "
            + so.getPid()));
      }  
    }

    for (HierarchieObjekt lauf : _hierarchie.getUntergeordneteObjekte(ho))
    {
      bestimmeKonkreteObjekte(lauf, konkreteObjekte, meldung);
    }
  }
  
  /**
   * Bestimmt die konkreten Systemobjekte, die gelscht werden, wenn ein Hierarchieobjekt gelscht werden soll, und fgt
   * sie an eine Liste an.
   * 
   * @param ho
   *          das zu lschende Hierarchieobjekt
   * @param konkreteObjekte
   *          die Liste konkreter Systemobjekte, an die die bestimmten Objekte angehngt werden
   */ 
  private void bestimmeKonkreteObjekte(HierarchieObjekt ho, Collection<SystemObject> konkreteObjekte)
  {
    bestimmeKonkreteObjekte(ho, konkreteObjekte, null);
  }

  /**
   * Bestimmt die konkreten Systemobjekte, die gelscht werden, wenn ein  oder mehrere Hierarchieobjekte gelscht werden 
   * sollen, und fgt sie an eine Liste an.
   * 
   * @param hierarchieObjekte
   *          die zu lschenden Hierarchieobjekte
   */ 
  public List<SystemObject> bestimmeKonkreteObjekte(Collection<HierarchieObjekt> hierarchieObjekte)
  {
    List<SystemObject> konkreteObjekte = new LinkedList<SystemObject>();
    
    for (HierarchieObjekt ho: hierarchieObjekte)
    {  
      bestimmeKonkreteObjekte(ho, konkreteObjekte);
    }
    
    return konkreteObjekte;
  }
  /**
   * Bestimmt die konkreten Systemobjekte, die gelscht werden, wenn ein Hierarchieobjekt gelscht werden soll, und fgt
   * sie an eine Liste an. Eine Multimeldung wird um eine entsprechende Statusmeldung ergnzt
   * 
   * @param ho
   *          das zu lschende Hierarchieobjekt
   * @param konkreteObjekte
   *          die Liste konkreter Systemobjekte, an die die bestimmten Objekte angehngt werden
   * @param meldung
   *          die Multimeldung, in die fr jedes gefundene konkrete Objekt eine Statusmeldung eingefgt wird
   */
  private void bestimmeVerboteneTypen(HierarchieObjekt ho, Collection<SystemObjectType> liste)
  {
    IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    Collection<SystemObjectType> typen = _hierarchieManager.loeschenOffeneReferenzenVerboten(hot);

    if (typen != null )
    {
      for (SystemObjectType type : typen)
      {
        if (!liste.contains(type))
        {
          liste.add(type);
        }
      }
    }

    for (HierarchieObjekt uho : _hierarchie.getUntergeordneteObjekte(ho))
    {
      bestimmeVerboteneTypen(uho, liste);
    }
  }


  /**
   * Bestimmt die offenen Referenzen, die enstehen, wenn eine Sammlung von Systemobjekten gelscht werden sollen. Fr
   * jede gefundene offene Referenz eine Multimeldung um eine Statusmeldung ergnzt
   * 
   * @param objekte
   *          die Liste der Systemobjekte, die gelscht werden sollen
   * @param typen
   *          Liste von Objekttypen, bei denen das Auftreten einer offenen Referenz unabhngig vom
   *          Referenzierungstyp zum Status Fehler fhrt
   * @param ausschlussListe
   *          Liste von Objekten, ind denen keine offenen Referenzen geprft werden sollen
   * @param verwendeZuLoeschendeObjekte
   *          Flagge, die anzeigt ob zu loeschende Objekte zur Prfung verwendet werden sollen          
   * @return eine Instanz vom Typ IStatus, die den Status und die Information ber die offenen Referenzen enthlt
   */
  public IStatus bestimmeOffeneReferenzen(Collection<SystemObject> objekte, Collection<SystemObjectType> typen,
      Collection<SystemObject> ausschlussListe, boolean verwendeZuLoeschendeObjekte)
  {
    MultiStatus result = new MultiStatus(Activator.PLUGIN_ID, Status.OK,
        "Folgende Referenzen sind dann nicht mehr gltig:", null);
    ConfigDataModel dataModel = _hierarchie.getDataModel();
    Collection<ConfigurationArea> konfigBereiche = dataModel.getAllConfigurationAreas().values();
    int offeneReferenzen = 0;

    ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel, konfigBereiche);

    for (ConfigurationArea kb : konfigBereiche)
    {
      Collection<SystemObject> systemObjekte = kb.getCurrentObjects();
      systemObjekte.addAll(kb.getNewObjects());

      Set<SystemObjectType> metaModellTypen = _metaModellTypen.get(kb);

      for (SystemObject so : systemObjekte)
      {
        if (so instanceof ConfigurationObject)
        {
          ConfigurationObject ko = (ConfigurationObject)so;
          if (!(ko.getNotValidSince() >= kb.getModifiableVersion()))
          // Objekt wird in zuknftiger Version nicht ungltig
          {
            SystemObjectType typ = ko.getType();

            // if (!_pidsMetaModellTypen1.contains(typ.getPid()))
            if (metaModellTypen == null || !metaModellTypen.contains(typ))
            {
              if (!verwendeZuLoeschendeObjekte)
              {
                if (objekte.contains(so)) // Prfe nicht zu lschende Objekte
                {
//                  System.out.println("------------------------> wird nicht geprft: " + so);
                  continue;
                }
              }
              if (ausschlussListe != null)
              {
                if (ausschlussListe.contains(so)) // Prfe AusschlussListe
                {
//                  System.out.println("------------------------> wird nicht geprft: " + so);
                  continue;
                }
              }
              for (AttributeGroupUsage atgu : ko.getUsedAttributeGroupUsages())
              {
                if (_atgMitReferenz.contains(atgu.getAttributeGroup()))
                {
                  Data data = ((ConfigSystemObject)so).getConfigurationData(atgu, lookup);
                  if (data != null)
                  {
                    offeneReferenzen += bestimmeOffeneReferenzen(ko, atgu, data, objekte, result, typen);
                  }
                }
              }
            }
          }
        }
      }
    }

    if (offeneReferenzen == 0)
    {
      result.add(new Status(Status.OK, Activator.PLUGIN_ID, "<keine>"));
    }

    return result;
  }

  /**
   * Bestimmt die offenen Referenzen, die enstehen, wenn eine Sammlung von Systemobjekten gelscht werden sollen. Fr
   * jede gefundene offene Referenz eine Multimeldung um eine Statusmeldung ergnzt
   * 
   * @param objekte
   *          die Liste der Systemobjekte, die gelscht werden sollen
   * @param typen
   *          Liste von Objekttypen, bei denen das Auftreten einer offenen Referenz unabhngig vom
   *          Referenzierungstyp zum Status Fehler fhrt
   * @param verwendeZuLoeschendeObjekte
   *          Flagge, die anzeigt ob zu loeschende Objekte zur Prfung verwendet werden sollen          
   * @return eine Instanz vom Typ IStatus, die den Status und die Information ber die offenen Referenzen enthlt
   */
  public IStatus bestimmeOffeneReferenzen(Collection<SystemObject> objekte, Collection<SystemObjectType> typen,
      boolean verwendeZuLoeschendeObjekte)
  {
    return bestimmeOffeneReferenzen(objekte, typen, null, verwendeZuLoeschendeObjekte);
  }
  
  /**
   * Bestimmt die offenen Referenzen, die enstehen, wenn eine Sammlung von Systemobjekten gelscht werden sollen. Fr
   * jede gefundene offene Referenz eine Multimeldung um eine Statusmeldung ergnzt
   * 
   * @param objekte
   *          die Liste der Systemobjekte, die gelscht werden sollen
   * @param verwendeZuLoeschendeObjekte
   *          Flagge, die anzeigt ob zu loeschende Objekte zur Prfung verwendet werden sollen          
   * @return eine Instanz vom Typ IStatus, die den Status und die Information ber die offenen Referenzen enthlt
   */
  public IStatus bestimmeOffeneReferenzen(Collection<SystemObject> objekte, boolean verwendeZuLoeschendeObjekte)
  {
    return bestimmeOffeneReferenzen(objekte, null, null, verwendeZuLoeschendeObjekte);
  }
  
//  /**
//   * Bestimmt die offenen Referenzen, die enstehen, wenn eine Sammlung von Systemobjekten gelscht werden sollen. Fr
//   * jede gefundene offene Referenz eine Multimeldung um eine Statusmeldung ergnzt
//   * 
//   * @param objekte
//   *          die Liste der Systemobjekte, die gelscht werden sollen
//   * @return eine Instanz vom Typ IStatus, die den Status und die Information ber die offenen Referenzen enthlt
//   */
//  public IStatus bestimmeOffeneReferenzen(Collection<SystemObject> objekte)
//  {
//    return bestimmeOffeneReferenzen(objekte, null, null, false);
//  }
    
  /**
   * Prft fr ein Systemobjekt, ob in den Daten einer Attributgruppe Referenzen auf Systemobjekte, die in einer Liste
   * bergeben werden, vorhanden sind. Fr jede gefundene Referenz wird eine Multimeldung um eine Statusmeldung bezgl.
   * offener Refernz ergnzt
   * 
   * @param ko
   *          das Systemobjekt, das zu prfen ist
   * @param atgu
   *          die zu prfende Attributgruppe
   * @param data
   *          der zu prfende Datensatz
   * @param konkreteObjekte
   *          Systemobjekt, fr die in den bergebenen Daten nach Referenzen gesucht wird
   * @param meldung
   *          die Multimeldung, in die fr jede gefundene Referenz eine Statusmeldung bezgl. offener Referenz eingefgt
   *          wird
   * @return die Anzahl der gefundenden offenen Referenzen im bergebenen Datensatz
   */
  private int bestimmeOffeneReferenzen(ConfigurationObject ko, AttributeGroupUsage atgu, Data data,
      Collection<SystemObject> konkreteObjekte, MultiStatus meldung, Collection<SystemObjectType> typen)
  {
    int result = 0;

    if (data.isPlain())
    {

      AttributeType attTyp = data.getAttributeType();
      if (attTyp instanceof ReferenceAttributeType)
      {

        SystemObject ref = data.asReferenceValue().getSystemObject();

        if (konkreteObjekte.contains(ref))
        {
          // System.out.println("--------> ATG " + atgu.getAttributeGroup().getPid() + " : " +
          // ((ReferenceAttributeType)attTyp).getReferenceType());

          result++;

          // Workarround bzgl. Fehler in der Konfiguration
          SystemObjectType type = ((SystemObject)ko).getType();

          if ((typen != null && typen.contains(type)) || !((ReferenceAttributeType)attTyp).isUndefinedAllowed())
          {
            meldung.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Nicht-optionale Referenz bei Objekttyp "
                + ko.getType().getPid() + " mit Pid " + ko.getPid() + " in ATG " + atgu.getAttributeGroup().getPid()
                + " und ATT " + /*attTyp.getPid()*/ data.getName()));
          }
          else
          {
            meldung.add(new Status(Status.WARNING, Activator.PLUGIN_ID, "Referenz bei Objekttyp "
                + ko.getType().getPid() + " mit Pid " + ko.getPid() + " in ATG " + atgu.getAttributeGroup().getPid()
                + " und ATT " + /*attTyp.getPid()*/ data.getName()));
          }          
        }
      }
    }
    else if (data.isList() || data.isArray())
    {
      Iterator it = data.iterator();

      while (it.hasNext())
      {
        Data elem = (Data)it.next();
        result += bestimmeOffeneReferenzen(ko, atgu, elem, konkreteObjekte, meldung, typen);
      }
    }

    return result;
  }

  /**
   * Bestimmt alle Attributgruppen, die ein referenzierendes Attribut enhalten
   * 
   * @return Menge aller Attributgruppen, die ein referenzierendes Attribut enhalten
   */
  private Set<AttributeGroup> bestimmeAtgMitReferenz()
  {
    Set<AttributeGroup> result = new HashSet<AttributeGroup>();
    ConfigDataModel dataModel = _hierarchie.getDataModel();

    List<SystemObjectType> typSpez = new Vector<SystemObjectType>();
    typSpez.add(dataModel.getType("typ.attributgruppe"));

    Collection<SystemObject> objekte = dataModel.getObjects(null, typSpez, ObjectTimeSpecification.valid());

    for (SystemObject obj : objekte)
    {
      assert (obj instanceof AttributeGroup);
      AttributeGroup atg = (AttributeGroup)obj;
      AttributeGroupUsage atgu = atg.getAttributeGroupUsage(dataModel.getAspect("asp.eigenschaften"));
      if (atgu != null && atgu.isConfigurating())
      {
        List<Attribute> attribute = atg.getAttributes();
        for (Attribute att : attribute)
        {
          if (isReferenz(att))
          {
            result.add(atg);
            break;
          }
        }
      }
    }

    return result;
  }

  /**
   * Bestimmt fr ein Attribut, ob es ein referenzierendes Attribut ist. Sub-Attribute werden rekursiv durchsucht
   * 
   * @param att
   *          das zu untersuchende Attribut
   * @return <code>true</code>, wenn das Attribut ein referenzierendes Attribut ist, <code>false</code> sonst
   */
  private boolean isReferenz(Attribute att)
  {
    AttributeType typ = att.getAttributeType();

    if (!att.isArray())
    {
      return (typ instanceof ReferenceAttributeType);
    }
    else
    {
      if (typ instanceof AttributeListDefinition)
      {
        for (Attribute attribut : ((AttributeListDefinition)typ).getAttributes())
        {
          if (isReferenz(attribut))
          {
            return true;
          }
        }
      }
      else if (typ instanceof ReferenceAttributeType)
      {
        return true;
      }
      else
      {
        return false;
      }

      return false;
    }
  }

  /**
   * Inititialisiert die Map _metaModellTypen
   */
  private void initialisiereMetaModellTypen()
  {
    ConfigDataModel dataModel = _hierarchie.getDataModel();

    for (ConfigurationArea kb : dataModel.getAllConfigurationAreas().values())
    {
      List<SystemObjectType> typSpez = new Vector<SystemObjectType>();

      for (String pid : PIDS_META_MODELL_TYPEN)
      {
        SystemObjectType typ = dataModel.getType(pid);
        if (typ != null)
        {
          typSpez.add(typ);
        }
      }

      Collection<SystemObject> metaObjekte = kb.getObjects(typSpez, ObjectTimeSpecification.valid());

      Set<SystemObjectType> set = new HashSet<SystemObjectType>();

      for (SystemObject obj : metaObjekte)
      {
        set.add(obj.getType());
      }

      if (set.size() > 1)
      {
        _metaModellTypen.put(kb, set);
      }
    }
  }

}
