/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.tkaUpdateManagement;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieManager;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Suchen mindestens eines Systemobjekts, das in einem Hierarchieobjekt als Elternobjekt in der zu durchsuchenden
 * Hierarchie zu den Objekten einer Liste von Systemobjekten in Frage kommt 
 * 
 * @author Sans
 */
public class FindePotentiellesElternObjekt extends HierarchieIterierer
{
  /**
   * Menge der Systemobjekttypen aller Systemobjekte, zu denen ein potentielles Elternobjekt gesucht wird
   */
  private Set<SystemObjectType> _typen = new HashSet<SystemObjectType>();
  
  /**
   * HierarchieManager-Instanz zum feststellen erlaubter Untertypen eines Objektes in der Hierarchie
   */
  private IHierarchieManager _hm;
  
  private boolean _einPotentiellesElternGefunden;
  
  /**
   * Wurde mindestens fr eines der bergenenen Systemobjekte ein potetielles Elternobjekt in der Hierarchie
   * gefunden?
   * 
   * @return true, wenn fr mindesten eines der Systemobjekte ein potetielles Elternobjekt in der Hierarchie vorkommt, 
   * false sonst
   */
  public boolean isEinPotentiellesElternGefunden()
  {
    return _einPotentiellesElternGefunden;
  }

  /**
   * Erzeugt eine neue Instanz und durchsucht die bergebene Hierarchie. Nach Durchlauf des
   * Konstruktors kann das Ergebnis mit isEinPotentiellesElternGefunden() abgefragt werden
   * 
   * @param liste die Liste der Systemobjekte, fr die gesucht wird
   * @param hierarchie die zu durchsuchende Hierarchie
   */
  public FindePotentiellesElternObjekt(List<SystemObject> liste, IHierarchieManager hm)
  {
    super(hm.getHierarchie());
    
    _hm = hm;
    
    for (SystemObject so: liste)
    {
      _typen.add(so.getType());
    }
    
    if (!_typen.isEmpty())
    {  
      alleObjekteIterieren();
    }  
  }
  
  @Override
  protected boolean iteration(Object objekt)
  {
    IHierarchieObjektTyp[] hotArray = _hm.getUntergeordneteHOTs(objekt);
     
    if (hotArray == null)
    {
      return true;
    }
    
    for (IHierarchieObjektTyp hot: _hm.getUntergeordneteHOTs(objekt))
    {
      List<String> typPids = hot.getTypen(); 
      int anzahl = typPids.size();
      
      if (anzahl == 1)
      {
        SystemObjectType typ = _hm.getHierarchie().getDataModel().getType(typPids.get(0));
        if (typ != null && _typen.contains(typ))
        {
          _einPotentiellesElternGefunden = true;
          return false;
        }
      }
      else if (anzahl > 1)
      {
        Set<SystemObjectType> typen = new HashSet<SystemObjectType>();
        
        for (String pid: typPids)
        {
          SystemObjectType typ = _hm.getHierarchie().getDataModel().getType(pid);
          if (typ != null)
          {
            typen.add(typ);
          }
        }
        
        if (typen.removeAll(_typen))
        {
          _einPotentiellesElternGefunden = true;
          return false;
        }
      }
    }
   
    return true;
  }
  
}
