/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkatls;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.MengeInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

public class DeUnterTyp  implements IHierarchieObjektTyp
{
	public final static String DE_OBER_TYP = "typ.de";
	
  /**
   * Name des HOT - wird in der Navigationsdarstellung verwendet
   */
  private String _name;
  
  /**
   * Die Systemobjekt-Typen, die von diesem HOT verwendet werden
   */
  private List<String> _typen = new Vector<String>();
  
  /**
   * Die Typinfos zu den Systemobjekttypen
   */
  private Map<String, TypInfo> _typInfos = new HashMap<String, TypInfo>();
  
  /**
   * Die Funktionsgruppe fr diesen DE-Typ
   */
  private int _fg;
  
  /**
   * bergeordnemter Hierarchieobjekttyp
   */
  private DeUnterTyp _fgRelevanterHOT;
  
  /**
   * Bestimmt den (bergeordneten) HOT, durch den die FG festgelegt ist
   * 
   * @return den HOT, durch den die FG festgelegt ist
   */
  public DeUnterTyp getFrgRelevantenHOT()
	{
		return _fgRelevanterHOT;
	}

	/**
   * Erzeugen eines neuen HOT_MQ auf Basis eines bestehenden HOT_TLS
   * Es werden alle Infos des bergeordnetenHOT fr den Systemobjekttyp oberTyp 
   * in die neu zu erzeugende Instanz fr den Sytemobjekttyp unterTyp bernommen.<br>
   * Die Erzeugung eines HOT auf diese Art mit mehr als einem Systemobjekttyp ist nicht
   * mglich
   * 
   * @param name Name des HOT
   * @param uebergeorneterHOT der HOT, der beerbt werden soll
   * @param HOT, durch den die FG festgelegt ist
   */
  public DeUnterTyp(String typ, int fg, DeUnterTyp fgRelevanterHOT)
  {
    TypInfo typInfo = HOT_TLS.DE.getTypInfo(DE_OBER_TYP);
    List<String> atgs = typInfo.getAtgs();
    AtgInfo atgInfo[] = new AtgInfo[atgs.size()];
    for (int i = 0; i < atgInfo.length; i += 1)
    {
    	atgInfo[i] = typInfo.getAtgInfo(atgs.get(i)); 
    }
    List<String> mengen = typInfo.getMengen();
    MengeInfo mengeInfo[] = new MengeInfo[mengen.size()];
    for (int i = 0; i < mengeInfo.length; i += 1)
    {
    	mengeInfo[i] = typInfo.getMengeInfo(mengen.get(i)); 
    }
    
    _fg = fg;
    _name = bestimmeName(typ);
    _typen.add(typ);
    _typInfos.put(typ, new TypInfo(typ, atgInfo, mengeInfo));
    _fgRelevanterHOT = fgRelevanterHOT == null ? this : fgRelevanterHOT;
  }
  
  private String bestimmeName(String typ)
	{
		String result = null;
		
		if (typ.startsWith(DE_OBER_TYP)) // Pattern typ.deXXX
		{
			int len = DE_OBER_TYP.length();
			if (typ.length() > len)
			{
				result = "DE_" + typ.substring(len);
			}
		}
		
		if (result == null || "".equals(result)) 
		{
			result = typ;
		}
		
		return result.toUpperCase();
	}	
  
  /**
   * Text-Reprsentation fr diesen HOT
   * 
   * @return den Namen des HOT - wird in der Navigationsdarstellung verwendet
   */
  @Override
  public String toString()
  {
    return getName();
  }
    
  /**
	 * @see IHierarchieObjektTyp#getName()
	 */
  public String getName()
  {
    return _name;
  }
  
  /**
	 * @see IHierarchieObjektTyp#getSymbolName()
	 */
  public String getSymbolName()
  {
  	return "DE";
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypen()
	 */
  public List<String> getTypen()
  {
    return Collections.unmodifiableList(_typen); 
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypInfo(String)
	 */
  public TypInfo getTypInfo(String typ)
  {
  	return _typInfos.get(typ);
  }
  
  /**
	 * @see IHierarchieObjektTyp#getTypInfo(SystemObjectType)
	 */
  public TypInfo getTypInfo(SystemObjectType sot)
  {
  	TypInfo result = getTypInfo(sot.getPid());
  	
  	if (result == null && !sot.isBaseType())
  	{
  		for (SystemObjectType superTyp: sot.getSuperTypes())
  		{
  			if (superTyp.isConfigurating())
  			{
  				 result = getTypInfo(superTyp);
  				 if (result != null)
  				 {
  					 break;
  				 }
  			}
  		}
  	}
  	
  	return result;
  }

  @Override
  public boolean isSystemObjektVorhanden()
  {
    return false;
  }
  
  public boolean isSystemObjektNichtAenderbar()
  {
  	return isSystemObjektVorhanden();
  }
  
  public int getFg()
  {
  	return _fg;
  }
}
